__title__ = 'MindsDB DuckDB handler'
__package_name__ = 'mindsdb_duckdb_handler'
__version__ = '0.0.1'
__description__ = 'MindsDB handler for DuckDB'
__author__ = 'Kamil Tyborowski'
__github__ = 'https://github.com/mindsdb/mindsdb'
__pypi__ = 'https://pypi.org/project/mindsdb/'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022 - mindsdb'
