__title__ = "MindsDB FaunaDB handler"
__package_name__ = "mindsdb_faunadb_handler"
__version__ = "0.0.1"
__description__ = "MindsDB handler for FaunaDB"
__author__ = "Meet Gor"
__github__ = "https://github.com/mindsdb/mindsdb"
__pypi__ = "https://pypi.org/project/mindsdb/"
__license__ = "MIT"
__copyright__ = "Copyright 2023 - mindsdb"
